import arrow
import datetime
import collections

_State = collections.namedtuple('_State', 'dates viewed_month rooms')

def parse_date(s):
    (year, month, day) = s.split('-')
    return datetime.date(year=int(year), month=int(month), day=int(day))

def format_date(date):
    return '{0.year}-{0.month}-{0.day}'.format(date)

def parse_month(s):
    (year, month) = s.split('-')
    return arrow.get(int(year), int(month), 1)

def format_month(date):
    return '{0.year}-{0.month}'.format(date)

class State(_State):
    @classmethod
    def from_request_args(Cls, args):
        today = datetime.date.today()
        if 'viewed_month' in args:
            viewed_month = parse_month(args['viewed_month'])
        else:
            viewed_month = arrow.get(year=today.year, month=today.year, day=1)
        if 'dates' in args:
            dates = tuple(sorted(map(parse_date, args['dates'].split())))
        else:
            dates = (today,)
        rooms = tuple(map(int, filter(bool, args.get('rooms', '').split())))
        return Cls(viewed_month=viewed_month, dates=dates, rooms=rooms)

    def to_request_args(self):
        return dict(
                viewed_month=format_month(self.viewed_month),
                dates=' '.join(map(format_date, self.dates)),
                rooms=' '.join(map(str, self.rooms)),
                )

    def show_prev_month(self):
        viewed_month = self.viewed_month.replace(months=-1)
        dates = self.dates
        rooms = self.rooms
        return self.__class__(viewed_month=viewed_month, dates=dates, rooms=rooms)

    def show_next_month(self):
        viewed_month = self.viewed_month.replace(months=+1)
        dates = self.dates
        rooms = self.rooms
        return self.__class__(viewed_month=viewed_month, dates=dates, rooms=rooms)

    def unselect_date(self, date):
        viewed_month = self.viewed_month
        dates = tuple(d for d in self.dates if d != date)
        rooms = self.rooms
        return self.__class__(viewed_month=viewed_month, dates=dates, rooms=rooms)

    def select_date(self, date):
        viewed_month = self.viewed_month
        if date in self.dates:
            dates = self.dates
        else:
            dates = self.dates + (date,)
        rooms = self.rooms
        return self.__class__(viewed_month=viewed_month, dates=dates, rooms=rooms)

    def update_room_selection(self, rooms):
        viewed_month = self.viewed_month
        dates = self.dates
        rooms = tuple(rooms)
        return self.__class__(viewed_month=viewed_month, dates=dates, rooms=rooms)



import math
import datetime
import collections

import arrow
import flask_babel
from babel.dates import format_date

QuarterHour = collections.namedtuple('QuarterHour', 'roomstates')
RoomState = collections.namedtuple('RoomState', 'rowspan period class_ description')

def format_hour(a):
    locale = flask_babel.get_locale()
    date = datetime.time(hour=a.hour, minute=a.minute)
    return format_date(date, 'H:mm', locale=locale)

def build_table(dates, room_calendars):
    roomstates = [[] for _ in range(0, 24*4)]
    for date in dates:
        arrow_date = arrow.get(date.year, date.month, date.day)
        for room_calendar in room_calendars:
            events = room_calendar.events.on(arrow_date)
            events = sorted(events, key=lambda e: e.begin)
            for (event, next_event) in zip([None] + events, events + [None]):
                if event is not None:
                    begin_time = event.begin.hour + event.begin.minute/60.
                    end_time = event.end.hour + event.end.minute/60.
                    duration_time = event.duration.seconds/3600

                    begin_fmt = format_hour(event.begin)
                    end_fmt = format_hour(event.end)

                    roomstates[int(begin_time*4.)].append(RoomState(
                        rowspan=math.ceil(duration_time*4.),
                        class_="reserved",
                        period='{} - {}'.format(begin_fmt, end_fmt),
                        description=event.name or '(no description)',
                        ))
                else:
                    end_time = 0

                if next_event is None:
                    next_begin_time = 24.
                else:
                    next_begin_time = next_event.begin.hour + next_event.begin.minute/60.
                for i in range(int(end_time*4.), int(next_begin_time*4.)):
                    roomstates[i].append(RoomState(
                        rowspan=1,
                        class_="unused",
                        period=None,
                        description=None,
                        ))

    return roomstates



import datetime
import calendar
import dateutil
import collections
import dateutil.relativedelta

import flask_babel
from babel.dates import format_date

def get_month_name(year, month):
    locale = flask_babel.get_locale()
    date = datetime.date(year=year, month=month, day=1)
    return format_date(date, 'MMMM', locale=locale)

CalendarMonth = collections.namedtuple('CalendarMonth', 'name number year_number weeks')
CalendarWeek = collections.namedtuple('CalendarWeek', 'number days')
CalendarDay = collections.namedtuple('CalendarDay', 'number is_current is_selected')

NO_DAY = CalendarDay(number=None, is_current=False, is_selected=False)

def is_current_day(year, month, day):
    try:
        date = datetime.date(year, month, day)
    except ValueError: # day is out of range for month
        return False
    else:
        return date == datetime.date.today()

def is_selected_day(year, month, day, selected_dates):
    try:
        date = datetime.date(year, month, day)
    except ValueError: # day is out of range for month
        return False
    else:
        return date in selected_dates

def get_calendar_month(year, month, selected_dates):
    (first_weekday, number_of_days) = calendar.monthrange(year, month)
    weeks = []
    days = [NO_DAY]*(first_weekday-1)
    for day in range(number_of_days+1):
        days.append(CalendarDay(number=day,
            is_current=is_current_day(year, month, day),
            is_selected=is_selected_day(year, month, day, selected_dates),
            ))
        if len(days) == 7:
            week_number = datetime.date(year, month, day).isocalendar()[1]
            weeks.append(CalendarWeek(number=week_number, days=days))
            days = []
    if days:
        days += [NO_DAY]*(7-len(days))
        weeks.append(CalendarWeek(number=week_number+1, days=days))
    while len(weeks) < 6:
        weeks.append(CalendarWeek(number=None, days=[NO_DAY]*7))
    return CalendarMonth(name=get_month_name(year, month), number=month, year_number=year, weeks=weeks)

def get_calendar_months(selected_dates, viewed_month):
    today = datetime.datetime.today()
    this_month = viewed_month
    last_month = this_month + dateutil.relativedelta.relativedelta(months=-1)
    next_month = this_month + dateutil.relativedelta.relativedelta(months=+1)
    months = [
            get_calendar_month(last_month.year, last_month.month, selected_dates),
            get_calendar_month(this_month.year, this_month.month, selected_dates),
            get_calendar_month(next_month.year, next_month.month, selected_dates),
            ]
    return months



import os
import json
import urllib
import datetime
import collections

import flask
from flask import request
from babel import negotiate_locale
import flask_babel
import ics

from .calendar import get_calendar_months
from .state import State
from .roomstate import build_table

AVAILABLE_LOCALES = ['fr', 'en']

app = flask.Flask(__name__)
app.config['CAS_AFTER_LOGIN'] = 'booking_view'
app.config.from_envvar('CALRESA_SETTINGS')
babel = flask_babel.Babel(app)

if app.config['CAS_SERVER']:
    from flask_cas import CAS, login_required
    cas = CAS(app)
else:
    cas = None
    login_required = lambda f: f

def render_template(*args, **kwargs):
    kwargs['url_for_view_prev_month'] = url_for_view_prev_month
    kwargs['url_for_view_next_month'] = url_for_view_next_month
    kwargs['url_for_unselect_date'] = url_for_unselect_date
    kwargs['url_for_select_date'] = url_for_select_date
    kwargs['url_for_room_selection'] = url_for_room_selection
    kwargs['url_for_change_rooms'] = url_for_change_rooms
    resp = flask.make_response(flask.render_template(*args, **kwargs))
    resp.headers['Content-type'] = 'application/xhtml+xml; charset=utf-8'
    return resp

def url_for_view_prev_month():
    state = State.from_request_args(request.args)
    state = state.show_prev_month()
    return flask.url_for('booking_view', **state.to_request_args())

def url_for_view_next_month():
    state = State.from_request_args(request.args)
    state = state.show_next_month()
    return flask.url_for('booking_view', **state.to_request_args())

def url_for_unselect_date(year, month, day):
    state = State.from_request_args(request.args)
    state = state.unselect_date(datetime.date(year, month, day))
    return flask.url_for('booking_view', **state.to_request_args())

def url_for_select_date(year, month, day):
    state = State.from_request_args(request.args)
    state = state.select_date(datetime.date(year, month, day))
    return flask.url_for('booking_view', **state.to_request_args())

def url_for_room_selection():
    state = State.from_request_args(request.args)
    return flask.url_for('room_selection', **state.to_request_args())

def url_for_change_rooms():
    state = State.from_request_args(request.args)
    return flask.url_for('change_rooms', **state.to_request_args())

@babel.localeselector
def get_locale():
    preferred = [x.replace('-', '_') for x in request.accept_languages.values()]
    return negotiate_locale(preferred, AVAILABLE_LOCALES)

_calendar_cache = {} # id -> (last_update, calendar)
def load_calendar(id_):
    global _calendar_cache
    path = os.path.join(app.config['ICS_DIR'], '{}.ics'.format(id_))
    last_update = os.stat(path).st_mtime
    if id_ not in _calendar_cache or _calendar_cache[id_][0] < last_update:
        with open(path, encoding='utf8') as fd:
            calendar = ics.Calendar(fd.read())
        _calendar_cache[id_] = (last_update, calendar)
        return calendar
    else:
        return _calendar_cache[id_][1]


@app.route('/')
@login_required
def booking_view():
    with open(app.config['NAMES_JSON']) as fd:
        room_names = json.load(fd)

    state = State.from_request_args(request.args)
    room_calendars = []
    for room in state.rooms:
        room_calendars.append(load_calendar(room))
    return render_template('booking_view.xhtml',
            rooms=[(n, room_names[str(n)]) for n in state.rooms],
            months=get_calendar_months(state.dates, state.viewed_month),
            selected_dates=state.dates,
            quarterhours=build_table(state.dates, room_calendars),
            )

@app.route('/select-rooms/')
@login_required
def room_selection():
    with open(app.config['NAMES_JSON']) as fd:
        room_names = json.load(fd)

    state = State.from_request_args(request.args)
    return render_template('room_selection.xhtml',
            selected_rooms=state.rooms,
            all_rooms=[(int(i), n) for (i,n) in room_names.items()],
            serialized_state=urllib.parse.urlencode(state.to_request_args()),
            )

@app.route('/select-rooms/form-target', methods={'POST'})
@login_required
def change_rooms():
    state = State.from_request_args(request.args)
    if 'update' in request.form:
        rooms = map(int, request.form.getlist('room'))
        state = state.update_room_selection(rooms)
    url = flask.url_for('booking_view', **state.to_request_args())
    return flask.redirect(url)



#!/usr/bin/env python3

from setuptools import setup

with open('requirements.txt') as fd:
    install_requires = fd.read().split()

setup(name='calresa',
      version='0.1',
      description='Room booking viewer',
      author='Valentin Lorentz',
      author_email='progval+calresa@progval.net',
      url='https://github.com/ProgVal/Calresa',
      packages=['calresa'],
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Web Environment',
          'Framework :: Flask',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: MIT License',
          'Natural Language :: English',
          'Natural Language :: French',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Topic :: Office/Business :: Scheduling',
          ],
     )
